<?php
// $Id: entity_node.inc,v 1.1.2.1 2009/08/14 15:41:27 pounard Exp $

/**
 * @file
 * Node entity for Yamm
 */

/**
 * Simple Entity node implementation
 */
class EntityNode extends Entity {

  /**
   * (non-PHPdoc)
   * @see www/sites/all/modules/custom/yamm/yamm_api/Entity#_objectLoad($identifier)
   */
  protected function _objectLoad($nid) {
    return node_load((int) $nid);
  }

  /**
   * (non-PHPdoc)
   * @see www/sites/all/modules/custom/yamm/yamm_api/Entity#_constructDependencies($object)
   */
  protected function _constructDependencies($node) {
    // Dependency on content type
    $this->_addDependency('content', $node->type);

    // Dependency on user, only if exists
    if ($node->uid && ($user = user_load($node->uid))) {
      $user_uuid = $this->_addDependency('user', $user->uid);
      $this->_setData('user', $user_uuid);
    }

    // Dependency on terms
    $terms = array();
    foreach ($node->taxonomy as $term) {
      $term_uuid = $this->_addDependency('term', $term->tid);
      $terms[] = $term_uuid;
    }
    $this->_setData('terms', $terms);
  }

  /**
   * Internal method shared between _save() and _update() methods.
   *
   * @param object &$node
   * @return void
   */
  private function __restoreData(&$node) {
    // Be sure to update current revision
    unset($node->vid);
    unset($node->revision);

    // Restore owner
    if ($user_uuid = $this->_getData('user')) {
      $node->uid = (int) EntityFactory::getIdentifierByUuid($user_uuid);
    }

    // Restore terms
    $node->taxonomy = array();
    foreach ($this->_getData('terms') as $term_uuid) {
      $tid = (int) EntityFactory::getIdentifierByUuid($term_uuid);
      $node->taxonomy[$tid] = $tid;
    }
  }

  /**
   * (non-PHPdoc)
   * @see www/sites/all/modules/custom/yamm/yamm_api/Entity#_save($object)
   */
  protected function _save($node) {
    unset($node->nid);
    $this->__restoreData($node);
    node_save($node);
    return $node->nid;
  }

  /**
   * (non-PHPdoc)
   * @see www/sites/all/modules/custom/yamm/yamm_api/Entity#_update($object, $identifier)
   */
  protected function _update($node, $nid) {
    $node->nid = (int) $nid;
    $this->__restoreData($node);
    node_save($node);
  }
}