<?php
// $Id: yamm_server.admin.inc,v 1.1.2.1 2009/08/14 15:41:28 pounard Exp $

/**
 * @file
 * Yamm server admin pages and forms.
 */

/**
 * Multistep handling for profile overview
 */
function yamm_server_profile_overview($form_state) {
  if (! empty($form_state) && isset($form_state['op']) && ! empty($form_state['profiles'])) {
    // Multistep
    return _yamm_server_profile_overview_confirm($form_state);
  }
  else {
    return _yamm_server_profile_overview();
  }
}

/**
 * Profile overview page
 */
function _yamm_server_profile_overview() {
  yamm_server_bootstrap_sync();

  $form = array();
  $form['#tree'] = TRUE;

  $form['options'] = array(
    '#type' => 'fieldset',
    '#title' => t('Operations'),
    '#collapsible' => FALSE);
  $form['options']['op'] = array(
    '#type' => 'select',
    '#options' => array(
      'default' => t('Set selected as default'),
      'add' => t('Add new item'),
      'delete' => t('Delete selected')),
    '#default_value' => 'default');
  $form['options']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Update'),
    '#submit' => array('yamm_server_profile_overview_update'));

  $form['profiles'] = array();

  $headers = array(
    array('data' => t('Default')),
    array('data' => t('Profile')),
    array('data' => t('Name')),
    array('data' => t('Operations'))
  );

  $element = 0;
  $profiles = SyncProfile::getList(10, $element);

  array_unshift($headers, theme('table_select_header_cell'));
  $form['profiles']['header'] = array('#type' => 'value', '#value' => &$headers);

  $checkboxes = array();
  $radios = array();

  foreach ($profiles as $profile => $name) {
    $checkboxes[$profile] = '';
    $radios[$profile] = '';
    $form['profiles']['profile'][$profile]
      = array('#value' => $profile);
    $form['profiles']['name'][$profile]
      = array('#value' => check_plain($name));
    $form['profiles']['operations'][$profile]
      = array('#value' => l(t('Edit'), 'admin/yamm/profile/edit/' . $profile, array('query' => array('destination' => $_GET['q']))));
  }

  $form['profiles']['checkboxes'] = array('#type' => 'checkboxes', '#options' => isset($checkboxes) ? $checkboxes : array());
  $form['profiles']['radios'] = array('#type' => 'radios', '#options' => isset($radios) ? $radios : array(), '#default_value' => SyncProfile::getDefaultProfile());
  $form['#theme'] = 'yamm_server_profile_overview';
  $form['footer'] = array('#type' => 'markup', '#value' => theme('pager', $element));

  return $form;
}

/**
 * Multistep confirm callback for profile overview
 */
function _yamm_server_profile_overview_confirm($form_state) {
  $form = array();

  $form['profiles'] = array(
    '#type' => 'value',
    '#value' => serialize($form_state['profiles']));

  switch ($form_state['op']) {
    case 'delete':
      $question = t('Are you sure you want to delete these profiles?');
      $description = t('They will be definitely removed. You won\'t be able to restore them.');
      $submit = '_yamm_server_profile_list_remove_submit';
      break;
  }

  $form = confirm_form(
    $form,
    $question,
    ($form_state['redirect'] ? $form_state['redirect'] : $_GET['q']),
    $description);

  $form['#submit'][] = $submit;

  return $form;
}

function _yamm_server_profile_list_remove_submit($form, &$form_state) {
  yamm_server_bootstrap_sync();

  $profiles = unserialize($form_state['values']['profiles']);

  foreach ($profiles as $profile) {
    SyncProfile::delete($name);
  }

  drupal_set_message(t('Selected profiles deleted'));
}

/**
 * Submit handler profile overview
 */
function yamm_server_profile_overview_update($form, &$form_state) {
  yamm_server_bootstrap_sync();
  $op = $form_state['values']['options']['op'];

  switch ($op) {
    case 'add':
      $path = 'admin/yamm/profile/add';
      $query = array('destination' => $_GET['q']);
      drupal_goto($path, $query);
      break;

    case 'default':
      if (! empty($form_state['values']['profiles']['radios'])) {
        drupal_set_message(t('Default profile updated.'));
        SyncProfile::setDefaultProfile($form_state['values']['profiles']['radios']);
      }
      break;

    case 'delete':
      // Multistep handling
      $form_state['rebuild'] = TRUE;
      $form_state['op'] = $op;
      $form_state['profiles'] = $form_state['values']['profiles']['checkboxes'];
      break;

    default:
      drupal_set_message(t('Not implemented yet'));
      break;
  }
}

/**
 * Edit/create profile form
 */
function yamm_server_profile($form_state, $profile = NULL) {
  yamm_server_bootstrap_sync();

  $form = array();

  if (! $profile) {
    $update = FALSE;
  }
  else {
    drupal_set_title('Edit profile');
    $update = $profile->getInternalName();
  }

  $form['update'] = array('#type' => 'value', '#value' => $update);

  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Profile name'),
    '#required' => TRUE,
    '#default_value' => ($profile instanceof SyncProfile ? $profile->getName() : ''));

  $form['internal_name'] = array(
    '#type' => 'textfield',
    '#title' => t('Internal name'),
    '#required' => TRUE,
    '#default_value' => ($profile instanceof SyncProfile ? $profile->getInternalName() : ''),
    '#description' => t('Must contains only alphanumeric caracters or _ and -'));

  $views = '';
  if ($profile instanceof SyncProfile) {
    foreach ($profile->getViews() as $view) {
      $views .= $view->name . "\n";
    }
  }

  $form['views'] = array(
    '#type' => 'textarea',
    '#default_value' => $views,
    '#title' => t('Views that export content'),
    '#required' => TRUE,
    '#description' => t('Type one view internal name per line'));

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => $update ? t('Update') : t('Save'));

  return $form;
}

function yamm_server_profile_validate($form, &$form_state) {
  yamm_server_bootstrap_sync();

  $internal = trim($form_state['values']['internal_name']);
  $name = trim($form_state['values']['name']);
  $views = trim($form_state['values']['views']);

  $profile = new SyncProfile();

  if (empty($name)) {
    form_set_error('name', t('You must set a name'));
  }

  $views_errors = array();

  foreach (explode("\n", $views) as $view_name) {
    $view_name = trim($view_name);
    if (! empty($view_name)) {
      try {
        $view = SyncProfile::loadView($view_name);
        SyncProfile::getViewEntityType($view);
      }
      catch (SyncProfileException $e) {
        $views_errors[] = $view_name;
      }
    }
  }

  if (! empty($views_errors)) {
    form_set_error('views', t('Views @views does not exist or are invalid', array(
      '@views' => implode(",", $views_errors))));
  }
}

function yamm_server_profile_submit($form, &$form_state) {
  yamm_server_bootstrap_sync();

  $internal = trim($form_state['values']['internal_name']);
  $name = trim($form_state['values']['name']);
  $views = trim($form_state['values']['views']);

  if ($form_state['values']['update']) {
    $profile = SyncProfile::load($form_state['values']['update']);
  }
  else {
    $profile = new SyncProfile();
  }

  $profile->setInternalName($internal);
  $profile->setName($name);

  $profile->purgeViews();

  foreach (explode("\n", $views) as $view_name) {
    $view_name = trim($view_name);
    if (! empty($view_name)) {
      $profile->addView($view_name);
    }
  }

  $profile->save();

  if ($form_state['values']['profiles']) {
    drupal_set_message(t('Profile @profile updated', array('@profile' => check_plain($profile->getName()))));
  }
  else {
    drupal_set_message(t('Profile @profile created', array('@profile' => check_plain($profile->getName()))));
  }

  $form_state['redirect'] = 'admin/yamm/profile';
}

/**
 * Edit entity settings for known types for profile form
 */
function yamm_server_profile_settings($form_state, $profile) {
  $form = array('#tree' => TRUE);

  $types = array();

  foreach (EntityFactory::getSupportedTypes() as $type => $desc) {
    try {
      $settings = $profile->getSettingsForType($type);

      if (! $settings instanceof EntitySettings) {
        $settings = EntityFactory::getEntitySettingsInstance($type);
      }

      $form[$type] = array(
        '#type' => 'fieldset',
        '#collapsible' => FALSE,
        '#title' => $desc['name']);

      $form[$type] += $settings->form();

      $types[] = $type;
    }
    catch (EntityClassNotFoundException $e) {
      // Silent error, it means our Entity does not implement the
      // EntitySettings class
    }
  }

  $form['profile'] = array(
    '#type' => 'value',
    '#value' => $profile->getInternalName());

  $form['types'] = array(
    '#type' => 'value',
    '#value' => $types);

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'));

  return $form;
}

function yamm_server_profile_settings_validate($form, &$form_state) {
  yamm_server_bootstrap_sync();

  foreach ($form_state['values']['types'] as $type) {
    $errors = EntityFactory::getEntitySettingsInstance($type)->formValidate($form_state['values'][$type]);

    if (! empty($errors)) {
      foreach ($errors as $field => $error) {
        form_set_error($type . '][' . $field, $error);
      }
    }
  }
}

function yamm_server_profile_settings_submit($form, &$form_state) {
  yamm_server_bootstrap_sync();

  $profile = SyncProfile::load($form_state['values']['profile']);

  foreach ($form_state['values']['types'] as $type) {
    $settings = EntityFactory::getEntitySettingsInstance($type);
    $settings->formSubmit($form_state['values'][$type]);

    $profile->addSettings($settings);
  }

  $profile->save();
}

/**
 * Multistep handling for client list
 */
function yamm_server_client_list($form_state) {
  if (! empty($form_state) && isset($form_state['op']) && ! empty($form_state['clients'])) {
    // Multistep
    return _yamm_server_client_list_confirm($form_state);
  }
  else {
    return _yamm_server_client_list_page($node);
  }
}

/**
 * Multistep confirm callback for client list
 */
function _yamm_server_client_list_confirm($form_state) {
  $form = array();

  $form['clients'] = array(
    '#type' => 'value',
    '#value' => serialize($form_state['clients']));

  switch ($form_state['op']) {
    case 'remove':
      $question = t('Are you sure you want to delete these clients?');
      $description = t('They will be definitely removed. You won\'t be able to restore them.');
      $submit = '_yamm_server_client_list_remove_submit';
      break;

    case 'suspend':
      $question = t('Are you sure you want to suspend these clients?');
      $description = t('These clients won\'t be synchronized anymore. You will be able to activate them back whenever you want.');
      $submit = '_yamm_server_client_list_suspend_submit';
      break;

    case 'activate':
      $question = t('Are you sure you want to activate these clients?');
      $description = t('These clients will be synchronized. You will be able to suspend them whenever you want.');
      $submit = '_yamm_server_client_list_activate_submit';
      break;
  }

  $form = confirm_form(
    $form,
    $question,
    ($form_state['redirect'] ? $form_state['redirect'] : $_GET['q']),
    $description);

  $form['#submit'][] = $submit;

  return $form;
}

function _yamm_server_build_in_statement($clients) {
  $args = array();
  $in = array();

  foreach ($clients as $name) {
    $in[] = "'%s'";
    $args[] = $name;
  }

  $where[] = "name IN (" . implode(",", $in) . ")";

  return array(implode(" AND ", $where), $args);
}

function _yamm_server_client_list_remove_submit($form, &$form_state) {
  $clients = unserialize($form_state['values']['clients']);
  list($where, $args) = _yamm_server_build_in_statement($clients);
  db_query("DELETE FROM {yamm_server_client} WHERE " . $where, $args);
}

function _yamm_server_client_list_suspend_submit($form, &$form_state) {
  $clients = unserialize($form_state['values']['clients']);
  list($where, $args) = _yamm_server_build_in_statement($clients);
  array_unshift($args, YAMM_STATUS_SUSPEND);
  db_query("UPDATE {yamm_server_client} SET status = %d WHERE " . $where, $args);
}

function _yamm_server_client_list_activate_submit($form, &$form_state) {
  $clients = unserialize($form_state['values']['clients']);
  list($where, $args) = _yamm_server_build_in_statement($clients);
  array_unshift($args, YAMM_STATUS_ACTIVATE);
  db_query("UPDATE {yamm_server_client} SET status = %d WHERE " . $where, $args);
}

/**
 * Client list page
 */
function _yamm_server_client_list_page() {
  $form = array();
  $form['#tree'] = TRUE;

  $form['nid'] = array(
    '#type' => 'value',
    '#value' => $node->nid);

  $form['options'] = array(
    '#type' => 'fieldset',
    '#title' => t('Operations'),
    '#collapsible' => FALSE);
  $form['options']['op'] = array(
    '#type' => 'select',
    '#options' => array(
      'add' => t('Add new item'),
      'remove' => t('Remove selected'),
      'suspend' => t('Suspend selected'),
      'activate' => t('Activate selected')),
    '#default_value' => 'add');
  $form['options']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Update'),
    '#submit' => array('yamm_server_client_list_update'));

  $form['clients'] = array();

  $headers = array(
    array(
      'field' => 'name',
      'data' => t('Name')),
    array('data' => t('Url')),
    array(
      'field' => 'username',
      'data' => t('Username')),
    array(
      'field' => 'status',
      'data' => t('Status')),
    array('data' => t('Operations'))
  );

  $element = 0;
  $ts = tablesort_init($headers);
  $clients = yamm_server_get_clients(10, $element, $ts['sql'], $ts['sort']);

  array_unshift($headers, theme('table_select_header_cell'));
  $form['clients']['header'] = array('#type' => 'value', '#value' => &$headers);

  $checkclients = array();

  foreach ($clients as &$client) {
    $checkclients[$client->name] = '';
    $form['clients']['name'][$client->name]
      = array('#value' => check_plain($client->name));
    $form['clients']['url'][$client->name]
      = array('#value' => check_plain($client->url));
    $form['clients']['username'][$client->name]
      = array('#value' => check_plain($client->username));
    $form['clients']['status'][$client->name]
      = array('#value' => theme('yamm_server_status', $client->status));
    $form['clients']['operations'][$client->name]
      = array('#value' => theme('yamm_server_client_operations', $client, $_GET['q']));
  }

  $form['clients']['checkboxes'] = array('#type' => 'checkboxes', '#options' => isset($checkclients) ? $checkclients : array());
  $form['#theme'] = 'yamm_server_client_list';
  $form['footer'] = array('#type' => 'markup', '#value' => theme('pager', $element));

  return $form;
}

/**
 * Submit handler client list page
 */
function yamm_server_client_list_update($form, &$form_state) {
  $op = $form_state['values']['options']['op'];

  switch ($op) {
    case 'add':
      $path = 'admin/yamm/client/add';
      $query = array('destination' => $_GET['q']);
      drupal_goto($path, $query);
      break;

    case 'remove':
    case 'suspend':
    case 'activate':
      // Multistep handling
      $form_state['rebuild'] = TRUE;
      $form_state['op'] = $op;
      $form_state['clients'] = $form_state['values']['clients']['checkboxes'];
      break;

    default:
      drupal_set_message(t('Not implemented yet'));
      break;
  }
}

/**
 * Add client form
 */
function yamm_server_client_edit_form($form_state, $client = NULL) {
  $form = array();

  if (! $client) {
    $client = new stdClass();
    $update = FALSE;
  }
  else {
    drupal_set_title('Edit client');
    $update = TRUE;
  }

  $form['update'] = array('#type' => 'value', '#value' => $update);

  if ($update) {
    $form['display_name'] = array(
      '#type' => 'textfield',
      '#title' => t('Internal name'),
      '#default_value' => $client->name,
      '#disabled' => $update);
    $form['name'] = array('#type' => 'value', '#value' => $client->name);
  }
  else {
    $form['name'] = array(
      '#type' => 'textfield',
      '#required' => TRUE,
      '#title' => t('Internal name'),
      '#description' => t('Only letters, numbers and _ or - are allowed.'),
      '#default_value' => $client->name);
  }

  $form['url'] = array(
    '#type' => 'textfield',
    '#required' => TRUE,
    '#title' => t('Client base url'),
    '#default_value' => $client->url);

  $form['credentials']['username'] = array(
    '#type' => 'textfield',
    '#title' => t('Username'),
    '#description' => t('Maintenance username for remote client. It must have the "launch sync jobs" permission.'),
    '#default_value' => $client->username);

  $form['credentials']['pass'] = array(
    '#type' => 'password',
    '#title' => t('Password'),
    '#description' => t('You have to type here the remote user password. Leave empty if you do not want to modify it.'));

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => $update ? t('Update') : t('Save'));

  return $form;
}

function yamm_server_client_edit_form_validate($form, &$form_state) {
  if (! preg_match('/^[a-zA-Z0-9\-_]+$/', $form_state['values']['name'])) {
    form_set_error('name', t('Invalid name'));
  }

  $exists = yamm_server_client_exists($form_state['values']['name']);
  $update = $form_state['values']['update'];

  if ($update) {
    if (! $exists) {
      form_set_error('name', t('Client does not exists'));
    }
  }

  else {
    if ($exists) {
      form_set_error('name', t('Client name already taken'));
    }
    if (empty($form_state['values']['pass'])) {
      form_set_error('pass', t('You must set a password'));
    }
  }

  if (empty($form_state['values']['username'])) {
    form_set_error('username', t('You must set credentials'));
  }
}

function yamm_server_client_edit_form_submit($form, &$form_state) {
  $update = $form_state['values']['update'];
  $pass = $form_state['values']['pass'];
  $name = $form_state['values']['name'];

  if ($update && empty($pass)) {
    $client = yamm_server_client_load($name);
    $pass = $client->pass;
  }
  else {
    $pass = md5($pass);
  }

  $client = array(
    'name' => $name,
    'url' => $form_state['values']['url'],
    'username' => $form_state['values']['username'],
    'pass' => $pass);

  yamm_server_client_save($client);
}


/**
 * Client hello test page
 */
function yamm_server_hello_page($client) {
  $output = '';

  module_load_include('inc', 'yamm_server', 'yamm_server.network');

  if (yamm_server_network_hello($client)) {
    drupal_set_message(t('Communication OK'));
    $output .= t('Server is up');
  }
  else {
    drupal_set_message(t('Unable to reach client'), 'error');
    $output .= t('Server seems to be down');
  }

  $output .= theme('yamm_server_page_back');

  return $output;
}

/**
 * Client overview page
 */
function yamm_server_client_overview($client) {
  $output = '';

  drupal_set_title(t('@name overview', array('@name' => check_plain($client->name))));

  $output .= drupal_get_form('yamm_server_client_operations_form', $client);
  $output .= theme('yamm_server_client_overwiew', $client);
  $output .= theme('yamm_server_page_back');

  return $output;
}

/**
 * Client operation form
 */
function yamm_server_client_operations_form($form_state, $client) {
  $form = array();

  $form['client'] = array('#type' => 'value', '#value' => check_plain($client->name));

  $form['operations'] = array(
    '#type' => 'fieldset',
    '#title' => t('Push method'),
    '#collapsible' => FALSE);

  $methods = yamm_api_push_methods();

  $form['operations']['oper'] = array(
    '#type' => 'select',
    '#options' => $methods,
    '#default_value' => array_shift(array_keys(yamm_api_push_methods())));

  $form['operations']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Push'));

  $form['#theme'] = 'yamm_server_client_operations_form';

  return $form;
}

function yamm_server_client_operations_form_submit($form, &$form_state) {
  $client = yamm_server_client_load($form_state['values']['client']);
  $op = $form_state['values']['oper'];

  if (yamm_api_push_method_exists($op)) {
    module_load_include('inc', 'yamm_server', 'yamm_server.network');

    if ($status = yamm_server_network_push($client, $op)) {
      switch ($status) {
        case YAMM_TRANSACTION_STATUS_WAITING:
          drupal_set_message(t('New transaction created, waiting for client to finish.'));
          break;
        case YAMM_TRANSACTION_STATUS_CANCELED:
          drupal_set_message(t('Client canceled transaction'), 'error');
          break;
        default:
          drupal_set_message(t('Unknown status returned by client (@id: @status)', array('@id' => $status, '@status' => yamm_api_get_status_localized_string($status))), 'error');
      }
    }
    else {
      drupal_set_message(t('Client not responding'), 'error');
    }
  }
  else {
    drupal_set_message(t('Unknown method, push not send'), 'error');
  }
}
